package gg.now.wallet.sdk.kotlinwalletdemo


import android.app.Activity
import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import gg.now.wallet.sdk.*
import gg.now.wallet.sdk.data.Nft
import gg.now.wallet.sdk.data.TransactionInfo
import gg.now.wallet.sdk.data.User
import gg.now.wallet.sdk.exception.SocketNotConnectedException
import gg.now.wallet.sdk.exception.WalletSdkNotInitialisedException
import gg.now.wallet.sdk.kotlinwalletdemo.common.Constants
import org.web3j.abi.datatypes.Bool
import timber.log.Timber


class MainActivityViewModel : ViewModel() {


    val cannyBalance = MutableLiveData("0.0")
    val userName = MutableLiveData<String>("")
    val avatarUrl = MutableLiveData<String>("")
    val isUserLoggedIn = MutableLiveData<Boolean>(false)
    val sessionTopic = MutableLiveData<String>("")
    val walletAddress = MutableLiveData<String>("")
    val txnHash = MutableLiveData<String>("")
    val isConnected = MutableLiveData<Boolean>(false)
    val error = MutableLiveData<String?>()


    inner class WalletDelegateHandler : NowGGWalletDelegate {
        override fun onConnectionStateChange(isAvailable: Boolean) {
            Timber.d("onConnectionStateChange: $isAvailable")
            isConnected.postValue(isAvailable)
        }

        override fun onError(error: String) {
            Log.e(javaClass.simpleName,"onError: $error")
            if (error == ErrorCodes.SESSION_EXPIRED) {
                isUserLoggedIn.postValue(false);
                this@MainActivityViewModel.error.postValue(error)
            }

        }

        override fun onSessionApproved(approvedSession: ApprovedSession, user: User) {
            Timber.d("onSessionApproved: " + approvedSession.accounts[0])
            sessionTopic.postValue(approvedSession.topic)
            isUserLoggedIn.postValue(true)
            userName.postValue(user.name)
            avatarUrl.postValue(user.avatar)
            walletAddress.postValue(approvedSession.accounts[0].split(":")[2])
            getTokenBalance()
        }

        override fun onSessionDelete(deletedSession: DeletedSession) {
            Timber.d("onSessionDelete: " + deletedSession.topic)

        }

        override fun onSessionRejected(rejectedSession: RejectedSession) {
            Timber.d("onSessionRejected: " + rejectedSession.topic)
        }

        override fun onSessionUpdate(updatedSession: UpdatedSession) {
            Timber.d("onSessionUpdate: " + updatedSession.topic)

        }

        override fun onTokenPurchaseError(error: String, extra: String?) {
            Timber.d("onTokenPurchaseError: error:$error, extra: $extra")

        }

        override fun onTokenPurchaseSuccess(result: SessionRequestResponse, extra: String?) {
            Timber.d("onTokenPurchaseSuccess: " + result.result)

            NowGGWalletSdkManager.getInstance().getTokenBalance(Constants.COIN_CONTRACT_ADDRESS,
                Constants.CAIP_CHAIN_ID,
                walletAddress.value!!,
                { amount ->
                    cannyBalance.postValue(
                        amount
                    )
                },
                { error ->
                    Timber.e(error)
                })
        }

        override fun onTransactionCompletedError(transactionInfo: TransactionInfo, extra: String?) {
            Timber.d("onTransactionCompletedError: " + transactionInfo.transactionHash + ", extra: " + extra)

        }

        override fun onTransactionCompletedSuccess(transaction: TransactionInfo, extra: String?) {
            Timber.d("onTransactionCompletedSuccess: " + transaction.transactionHash + ", extra: " + extra)

        }

        override fun onTransactionSendError(error: String, extra: String?) {
            Timber.d("onTransactionSendError: $error, extra: $extra")
        }

        override fun onTransactionSendSuccess(
            txnHash: String, caipChainId: String, extra: String?
        ) {
            Timber.d("onTransactionSendSuccess: $txnHash, extra: $extra")
            this@MainActivityViewModel.txnHash.postValue("https://mumbai.polygonscan.com/tx/$txnHash")
        }

    }


    fun initialiseWalletSdk(activity: MainActivity) {
        NowGGWalletSdkManager.getInstance()
            .initialize(activity, Constants.API_KEY, Constants.APP_ID, WalletDelegateHandler(), {
                Log.d(javaClass.simpleName, "Wallet sdk intialisation success")
            }, {
                this.error.postValue("Wallet sdk intialisation failed");
                Log.d(javaClass.simpleName, "Wallet sdk intialisation failed")
            })
    }

    fun connectWithWallet(activity: Activity) {
        try {
            NowGGWalletSdkManager.getInstance().connect(Constants.CAIP_CHAIN_ID, activity, {
                // on pairing invoked
                Log.d(javaClass.simpleName,"Pairing invoked success")
            }, { error ->
                // on error
                this.error.postValue("error connecting to wallet, Please retry or try changing your network");
            })
        } catch (ex: WalletSdkNotInitialisedException) {
            error.postValue("Please initialise the sdk first")
        } catch (ex: SocketNotConnectedException) {
            error.postValue("Please check your connectivity")
        }
    }

    fun getTokenBalance() {

        NowGGWalletSdkManager.getInstance().getTokenBalance(Constants.COIN_CONTRACT_ADDRESS,
            Constants.CAIP_CHAIN_ID,
            walletAddress.value!!,
            { balance ->
                //you can update user about the balance here
                cannyBalance.postValue(balance)
            },
            { error ->
                //handle error
                Log.d(javaClass.simpleName, "error fetching while balance:$error")

            })
    }


    fun buyToken(amount: Int) {
        val extra = "BuyToken"
        try {
            NowGGWalletSdkManager.getInstance().buyToken(
                Constants.COIN_CONTRACT_ADDRESS, amount.toString(), Constants.CAIP_CHAIN_ID, extra
            )
        } catch (ex: WalletSdkNotInitialisedException) {
            error.postValue("Please initialise the sdk first")
        } catch (ex: SocketNotConnectedException) {
            error.postValue("Please check your connectivity")
        }
    }

    fun sendTransaction() {
        val toAddress: String = Constants.COIN_CONTRACT_ADDRESS
        val nativeTokenValue = "0x0"
        //nftData is encoded method name with parameters of the NFT contract
        val nftData =
            "0x095ea7b3000000000000000000000000ff9f4c235d1508176cbe99891585e86b8fd3f09100000000000000000000000000000000000000000000001043561a8829300000"
        val extra = "froggy"
        try {
            NowGGWalletSdkManager.getInstance().sendTransaction(
                toAddress, nativeTokenValue, nftData, Constants.CAIP_CHAIN_ID, extra
            )
        } catch (ex: WalletSdkNotInitialisedException) {
            error.postValue("Please initialise the sdk first")
        } catch (ex: SocketNotConnectedException) {
            error.postValue("Please check your connectivity")
        }
    }

    fun getNfts() {

        walletAddress.value?.let {
            NowGGWalletSdkManager.getInstance()
                .getNfts(Constants.CAIP_CHAIN_ID, it, { nfts: List<Nft> ->
                    //todo: handle the list of nfts
                }, { error ->
                    //todo: handle error
                    Log.d(javaClass.simpleName, "error fetching while nfts: $error")

                })
        }
    }


}